import streamlit as st
import pandas as pd
from utils import *
from modules import *
import os
import numpy as np
import altair as alt
import plotly.graph_objects as go

absolute_path = os.path.abspath(__file__)
path = os.path.dirname(absolute_path)

ipl_ball    =  pd.read_csv(path+'/2008_2021_updated_ball.csv')
ipl_match   =  pd.read_csv(path+'/2008_2021_data_matches.csv')
season_list = ['2007/08','2009','2009/10','2011','2012','2013','2014','2015','2016','2017','2018','2019','2020/21','2021']
season_dict = {2008:'2007/08',2009:'2009',2010:'2009/10',2011:'2011',2012:'2012',2013:'2013',2014:'2014',2015:'2015',2016:'2016',2017:'2017',2018:'2018',2019:'2019',2020:'2020/21',2021:'2021'}
team_dict   =  { 'Delhi Capitals':'Delhi Daredevils' , 'Punjab Kings':'Kings XI Punjab' }

GRID  = True
WIDTH = 0

def formatt(df):
    temp = []
    for i in df.columns:
        if i in  ['SR', 'Avg', 'Eco','Win Percent']:
            temp.append(i)
    return df.style.format(subset=temp, formatter="{:.2f}")
    

def player_career():
    st.title('Player Career')
    player = st.sidebar.selectbox('Player', get_player_name(ipl_ball))
    bat  = pd.DataFrame(get_run( ipl_ball, [player] ))
    bat  = bat.drop(['batsman'], axis = 1)
    bat['M'] = 'IPL' 
    bat  = bat.set_index('M')
    bowl = pd.DataFrame(get_wicket( ipl_ball, [player]  ))
    bowl = bowl.drop(['bowler'], axis = 1)
    bowl['M'] = 'IPL' 
    bowl  = bowl.set_index('M')

    st.subheader('Batting Career')
    bat['Runs'] = bat.apply(lambda x: "{:,}".format(x['Runs']), axis=1)

    st.table(formatt(bat))

    st.subheader('Bowling Career')
    st.table(formatt(bowl))

    result = pd.DataFrame()
    for i in season_list:
        match = ipl_match[ipl_match['season'] == i]
        id = list(match['id'].unique())
        ball = ipl_ball[ipl_ball['id'].isin(id)]
        temp = get_run(ball, batsman = [player], choice = ['Innings','Runs','HS'])
        temp['year'] = i
        result = pd.concat([result,pd.DataFrame(temp)])
    st.subheader('Yearly Performance')
    result =  result.drop(['batsman'], axis = 1)
    c =  alt.Chart(result).mark_trail().encode(
        x='year:T',
        y='Runs:Q',
        size = 'Runs:Q',
        tooltip=['Runs:Q']
    ).configure_axis(
    grid= GRID
    ).configure_view(
        strokeWidth= WIDTH
    ).interactive()

    st.altair_chart(c, use_container_width=True)
    result_bat = result.set_index('year')

    result = pd.DataFrame()
    for i in season_list:
        match = ipl_match[ipl_match['season'] == i]
        id = list(match['id'].unique())
        ball = ipl_ball[ipl_ball['id'].isin(id)]
        temp = get_wicket(ball, bowler= [player], choice = ['Innings','Wickets','BBI'])
        temp['year'] = i
        result = pd.concat([result,pd.DataFrame(temp)])

    result =  result.drop(['bowler'], axis = 1)
    
    c =  alt.Chart(result).mark_trail().encode(
        x='year:T',
        y='Wickets:Q',
        size = 'Wickets:Q',
        tooltip = ['Wickets:Q'],
        color=alt.value("#FFAA00")
    ).configure_axis(
    grid= GRID
    ).configure_view(
        strokeWidth= WIDTH
    ).interactive()
    st.altair_chart(c, use_container_width=True)
    result_bowl = result.set_index('year')


    result = pd.merge(result_bat, result_bowl, how = 'outer', left_on = ['year'], right_on = ['year'])
    result = result[ ~ ((result['Innings_x'] == 0) & (result['Innings_y'] == 0))]
    result = result.rename(columns = {'Innings_x':'Innings Bat' ,'Innings_y':'Innings Bowl' })
    st.table(formatt(result))


def sesonal_stat():
    st.title('Sesonal Stats')
    result = pd.DataFrame()
    choice = ['Innings']
    player_type  = st.sidebar.selectbox('Player Type', ['Batsman','Bowler'] )
    
    if player_type == 'Batsman':
        option = ['Runs','Six','Four','Hundered','Fifty','BF','Avg','SR']
        flag  = st.sidebar.selectbox('Category', option )
        choice.append('Runs')
        if flag != 'Runs':
           choice.append(flag)
    else:
        option =  ['Wickets','Balls','Runs','BBI','Avg','SR','Eco','5W','BBI']
        flag  = st.sidebar.selectbox('Category', option )
        choice.append('Wickets')
        if flag != 'Wickets':
           choice.append(flag)
        

    limit = st.sidebar.slider('Top', 1, 10)
    start_year, end_year = st.sidebar.select_slider("Year", options = [2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021] , value = (2008,2021))

    temp = []
    for i in option:
        if i not in choice:
            temp.append(i)
    column = st.sidebar.multiselect('Cloumns',temp)

    for i in column:
        choice.append(i)

    innings = st.sidebar.number_input('Innings', step = 0)
    innings = None if innings == 0 else innings

    if 'Wickets' in choice:
        wickets = st.sidebar.number_input('Wickets', step = 0)
        wickets = None if wickets == 0 else wickets
    else:
        runs    = st.sidebar.number_input('Runs', step = 0)
        runs    = None if runs == 0 else runs

    for i in range(start_year, end_year+1):
        i = season_dict[i]
        match = ipl_match[ipl_match['season'] == i]
        id = list(match['id'].unique())
        ball = ipl_ball[ipl_ball['id'].isin(id)]
        if player_type == 'Batsman' :
            temp = get_run(ball,flag = flag, choice = choice, limit = limit, innings = innings, runs = runs)
        else:
            asc = True if flag in ['SR','Eco','Avg'] else False
            temp = get_wicket(ball,flag = flag, choice = choice, limit = limit, innings = innings, wickets = wickets, asc = asc )
        temp['year'] = i
        result = pd.concat([result,pd.DataFrame(temp)])
    result =  result.reset_index()
    result = result.rename(columns = {'index':'Position'})
    result['Position'] += 1
    if player_type == 'Batsman':
        choice.insert(0,'batsman')
    else:
        if 'BBI' in choice:
            choice.remove('BBI')
        choice.insert(0,'bowler')
        if flag == 'BBI':
            flag = 'Wickets'

    c = alt.Chart(result).mark_bar().encode(
        x=alt.X(flag),
        y='year',
        tooltip = choice,
    color = alt.Color('Position', scale=alt.Scale(scheme='redyellowgreen'))
    ).configure_axis(
        grid=False
    ).configure_view(
            strokeWidth= WIDTH
    )
    st.altair_chart(c,use_container_width=True )
    result = result.set_index('year')
    st.table(formatt(result))


def overall_stat():
    st.title('Overall Stats')
    result = pd.DataFrame()
    choice = ['Innings']
    player_type  = st.sidebar.selectbox('Player Type', ['Batsman','Bowler'] )
    
    if player_type == 'Batsman':
        option = ['Runs','Six','Four','Hundered','Fifty','BF','Avg','SR']
        flag  = st.sidebar.selectbox('Category', option )
        choice.append('Runs')
        if flag != 'Runs':
           choice.append(flag)
    else:
        option =  ['Wickets','Balls','Runs','BBI','Avg','SR','Eco','5W','BBI']
        flag  = st.sidebar.selectbox('Category', option )
        choice.append('Wickets')
        if flag != 'Wickets':
           choice.append(flag)
        

    limit = st.sidebar.slider('Top', 1, 20,10)

    temp = []
    for i in option:
        if i not in choice:
            temp.append(i)
    column = st.sidebar.multiselect('Cloumns',temp)

    for i in column:
        choice.append(i)

    innings = st.sidebar.number_input('Innings', step = 0)
    innings = None if innings == 0 else innings

    if 'Wickets' in choice:
        wickets = st.sidebar.number_input('Wickets', step = 0)
        wickets = None if wickets == 0 else wickets
    else:
        runs    = st.sidebar.number_input('Runs', step = 0)
        runs    = None if runs == 0 else runs

    
    if player_type == 'Batsman' :
        temp = get_run(ipl_ball,flag = flag, choice = choice, limit = limit, innings = innings, runs = runs)
        get_player_runs(temp['batsman'].unique())
    else:
        asc = True if flag in ['SR','Eco','Avg'] else False
        temp = get_wicket(ipl_ball,flag = flag, choice = choice, limit = limit, innings = innings, wickets = wickets, asc = asc )
        get_player_wickets(temp['bowler'].unique())
    result = pd.concat([result,pd.DataFrame(temp)])
    result =  result.reset_index()
    result = result.rename(columns = {'index':'Position'})
    result['Position'] += 1
    result = result.set_index('Position')
    
    st.table(formatt(result))

    if player_type == 'Batsman':
        c = alt.Chart(result).mark_circle( color='#EA484E').encode(
            alt.X('batsman', scale=alt.Scale(zero=False)),
            alt.Y('Innings', scale=alt.Scale(zero=False, padding=1)),
            tooltip = ['Runs']
        ).configure_axis(
        grid=GRID
        ).configure_view(
            strokeWidth= WIDTH
        ).encode(
        size=alt.Size('Runs', scale=alt.Scale(domain=[3000,5000]))
        ).interactive()
        st.altair_chart(c, use_container_width=True)
    else:
        c = alt.Chart(result).mark_circle( color='#EA484E').encode(
            alt.X('bowler', scale=alt.Scale(zero=False)),
            alt.Y('Innings', scale=alt.Scale(zero=False, padding=1)),
            tooltip = ['Wickets']
        ).configure_axis(
        grid=GRID
        ).configure_view(
            strokeWidth= WIDTH
        ).encode(
        size=alt.Size('Wickets', scale=alt.Scale(domain=[100,150]))
        ).interactive()
        st.altair_chart(c, use_container_width=True)


def one_vs_one():
    st.title('One Vs One')
    all_player = get_player_name(ipl_ball)
    player = st.sidebar.selectbox('Player', all_player)
    type_ = st.sidebar.selectbox('Vs', ['Team','Player'])

    if type_ == 'Player':
        player_removed = []
        for i in all_player:
            if i != player:
                player_removed.append(i)
        vs = st.sidebar.selectbox(type_,player_removed)
        vs_data = []
        vs_data.append(vs)

    else:
        team = []
        for i in list(ipl_ball['batting_team'].unique()):
            if i not in team_dict.values():
                team.append(i)

        vs = st.sidebar.selectbox(type_,team)
        vs_data = []
        vs_data.append(vs)
        if vs in team_dict.keys():
            vs_data.append(team_dict[vs])

    if type_ == 'Team':
        df_bat = get_team_data(vs_data, ipl_ball, 'batsman')
        df_bat = pd.DataFrame(get_run(df_bat,[player] ))
        df_bat = df_bat.drop(['batsman'], axis = 1)
        df_bat['M'] = 'IPL' 
        df_bat  = df_bat.set_index('M')
        st.subheader('Batting Stats')
        st.caption(player+' Vs '+vs_data[0])
        st.table(formatt(df_bat))

        df_bowl = get_team_data(vs_data, ipl_ball, 'bowler')
        df_bowl = pd.DataFrame(get_wicket(df_bowl,[player] ))
        df_bowl = df_bowl.drop(['bowler'], axis = 1)
        df_bowl['M'] = 'IPL' 
        df_bowl  = df_bowl.set_index('M')
        st.subheader('Bowling Stats')
        st.caption(player+' Vs '+vs_data[0])
        st.table(formatt(df_bowl))

    else:
        df_bat = get_player_data(vs_data, ipl_ball, 'bowler')
        df_bat = pd.DataFrame(get_run(df_bat,[player] ))
        df_bat = df_bat.drop(['batsman'], axis = 1)
        df_bat['M'] = 'IPL' 
        df_bat  = df_bat.set_index('M')
        st.subheader('Batting Stats')
        st.caption(player+' Vs '+vs_data[0])
        st.table(formatt(df_bat))

        df_bowl = get_player_data([player], ipl_ball, 'batsman')
        df_bowl = pd.DataFrame(get_wicket(df_bowl,vs_data ))
        df_bowl = df_bowl.drop(['bowler'], axis = 1)
        df_bowl['M'] = 'IPL' 
        df_bowl  = df_bowl.set_index('M')
        st.subheader('Bowling Stats')
        st.caption(vs_data[0]+' Vs '+player)
        st.table(formatt(df_bowl))


def over_stats():
    type_ = st.sidebar.selectbox('Type', ['Runs','Wickets','Six','Four','SR'])
    balls = 1
    if type_ in ['SR']:
        balls    = st.sidebar.number_input('BF', step = 0, min_value = 1)

    data  = best_in_over(ipl_ball, type_, balls)
    data  =  data.reset_index()
    data  = data.rename(columns = {'index':'Over'})
    data['over'] += 1 
    st.title('Over Stats')
    hide_table_row_index = """
            <style>
            tbody th {display:none}
            .blank {display:none}
            </style>
            """

    st.markdown(hide_table_row_index, unsafe_allow_html=True)
    choice = []
    choice.append('bowler' if type_ == 'Wickets' else 'batsman')

    choice.append(type_)
    c = alt.Chart(data).mark_area(
            line={'color':'darkgreen'},
            color=alt.Gradient(
                gradient='linear',
                stops=[alt.GradientStop(color='white', offset=0),
                    alt.GradientStop(color='darkgreen', offset=1)],
                x1=1,
                x2=1,
                y1=1.2,
                y2=0
            )
        ).encode(
            x = alt.X('over',scale = alt.Scale(zero=False)),
            y = '{0}'.format(type_),
            tooltip = choice
        ).configure_axis(
            grid=False
        ).configure_view(
            strokeWidth= WIDTH
        )
    st.altair_chart(c,use_container_width=True)
    st.table(formatt(data))


def win_predict_player():
    type_  = st.sidebar.selectbox('Type', ['Batting','Bowling'])
    team = []
    for i in list(ipl_ball['batting_team'].unique()):
        if i not in team_dict.values():
            team.append(i)
    team_1 = st.sidebar.selectbox('Team', team)
    team_data = []
    team_data.append(team_1)
    if team_1 in team_dict.keys():
        team_data.append(team_dict[team_1])

    if type_ == 'Batting':
        runs    = st.sidebar.number_input('Runs', step = 0, min_value = 30)
        batsman = list(ipl_ball[ipl_ball['batting_team'].isin(team_data)]['batsman'].unique())
        batsman.insert(0,None)
        player  = st.sidebar.selectbox('Player',batsman)
        opp = []
        for i in list(ipl_ball['batting_team'].unique()):
            if i not in team_dict.values() and i not in team_data:
                opp.append(i)
        opp.insert(0,None)
        opp_inp   =  st.sidebar.selectbox('Opponent',opp)
        opp_team = []
        opp_team.append(opp_inp)
        if opp_inp in team_dict.keys():
            opp_team.append(team_dict[opp_inp])

        venue     = list(ipl_match['venue'].unique())
        venue.insert(0,None)
        venue_inp = st.sidebar.selectbox('Venue',venue)
        innings   = st.sidebar.slider('Minimum Innings',1,10)

        win,result = decide_batsman(ipl_ball, ipl_match, team_data, player = player, runs = runs, opp = opp_team ,venue = venue_inp, thres = innings)
    else:
        wickets    = st.sidebar.number_input('Wickets', step = 0, min_value = 2 )
        bowler = list(ipl_ball[ipl_ball['bowling_team'].isin(team_data)]['bowler'].unique())
        bowler.insert(0,None)
        player  = st.sidebar.selectbox('Player',bowler)
        opp = []
        for i in list(ipl_ball['bowling_team'].unique()):
            if i not in team_dict.values() and i not in team_data:
                opp.append(i)
        opp.insert(0,None)
        opp_inp   =  st.sidebar.selectbox('Opponent',opp)
        opp_team = []
        opp_team.append(opp_inp)
        if opp_inp in team_dict.keys():
            opp_team.append(team_dict[opp_inp])

        venue     = list(ipl_match['venue'].unique())
        venue.insert(0,None)
        venue_inp = st.sidebar.selectbox('Venue',venue)
        innings   = st.sidebar.slider('Minimum Innings',1,10)
   
        win,result = decide_bowler(ipl_ball, ipl_match, team_data, player = player, wickets = wickets, opp = opp_team ,venue = venue_inp, thres = innings)

    source = pd.DataFrame({"category": ['Win', 'Lost'], "value": [win, 1- win]})

    st.title('Win Percent')
    fig = go.Figure(go.Indicator(
    mode = "gauge+number",
    value = int(win*100),
    domain = {'x': [0, 0.8], 'y': [0, 1]},
    gauge = {'axis':{'range':[0,100]}, }))
    st.plotly_chart(fig)
    st.table(formatt(result))

    fig.update_layout(xaxis = {'range': [0, 100]})



def get_player_runs(batsman):
    result = pd.DataFrame()
    for i in season_list:
        match = ipl_match[ipl_match['season'] == i]
        id   = list(match['id'].unique())
        ball = ipl_ball[ipl_ball['id'].isin(id)]
        ball = ball[ball['batsman'].isin(batsman)]
        temp = ball.groupby(['batsman']).agg({'batsman_runs':'sum'})
        temp['year'] = i
        temp = temp.rename(columns = {'batsman_runs':'Runs'})
        result = pd.concat([result,pd.DataFrame(temp)])
    result = result.reset_index()
    c = alt.Chart(result).mark_rect().encode(
        x = 'batsman:O',
        y = 'year:O',
        color = alt.Color('Runs:Q', scale=alt.Scale(scheme='reds')),
        tooltip = ['Runs:Q']
    ).configure_axis(
        grid=False
    ).configure_view(
            strokeWidth= WIDTH
    )
    st.altair_chart(c,  use_container_width=True)


def get_player_wickets(bowler):
    result = pd.DataFrame()
    for i in season_list:
        match = ipl_match[ipl_match['season'] == i]
        id   = list(match['id'].unique())
        ball = ipl_ball[ipl_ball['id'].isin(id)]
        ball = ball[(ball['is_wicket'] == 1) & (ball['dismissal_kind'].isin(['caught','bowled','lbw','stumped','caught and bowled','hit wicket']))]
        ball = ball[ball['bowler'].isin(bowler)]
        temp = ball.groupby(['bowler']).agg({'is_wicket':'sum'})
        temp = temp.rename(columns = {'is_wicket':'Wickets'})
        temp['year'] = i
        result = pd.concat([result,pd.DataFrame(temp)])
    result = result.reset_index()
    c = alt.Chart(result).mark_rect().encode(
        x = 'bowler:O',
        y = 'year:O',
        color = alt.Color('Wickets:Q', scale=alt.Scale(scheme='reds')),
        tooltip = ['Wickets:Q']
    ).configure_axis(
        grid=False
    ).configure_view(
            strokeWidth= WIDTH
    )
    st.altair_chart(c,  use_container_width=True)


import pandas as pd
import numpy as np

def get_start_end(player, ball, match):
    temp   =  match[['id','date']]
    flag   =  ball[['id','bowler']]
    player =  bbl_ball[bbl_ball['bowler'] == bowler ]
    final  =  pd.merge(temp,flag,how='inner',left_on=['id'],right_on=['id'])
    return final['date'].min(),final['date'].max()

def get_player_name(ball):
    return np.unique(ball[['batsman', 'bowler','non_striker']].values) 

def get_wicket(ball,bowler = None, flag = 'Wickets', innings = None, wickets = None, choice = None, limit = 3,  asc = False):
    
    player                  = ball[ball['bowler'].isin(bowler)] if bowler else ball
    wicket                  = len(player[(player['is_wicket'] == 1) & (player['dismissal_kind'].isin(['caught','bowled','lbw','stumped','caught and bowled','hit wicket']))])

    temp_wicket             = player[(player['is_wicket'] == 1) & (player['dismissal_kind'].isin(['caught','bowled','lbw','stumped','caught and bowled','hit wicket']))].groupby(['bowler']).agg({'is_wicket':'sum'}).astype(int)
    
    if len(temp_wicket) == 0:
        tt = pd.DataFrame(columns = [ 'bowler','Innings','Wickets','Balls','Runs','BBI','Avg','SR','Eco','5W' ] )
        tt = tt[choice] if choice else tt[['Innings','Wickets','Balls','Runs','BBI','Avg','SR','Eco','5W']]
        tt['bowler'] = bowler
        tt['BBI'] = '-'
        return tt.fillna(0) 
    
    temp_wicket['Innings']  = player.groupby(['bowler']).agg({'id':'nunique'})
    temp_wicket['Balls']    = player[~player['extras_type'].isin(['wides','noballs','penalty'])].groupby(['bowler']).agg({'id':'count'})
    temp_wicket['Runs']     = player[ ~ player['extras_type'].isin(['legbyes','byes','penalty'])].groupby(['bowler']).agg({'total_runs':'sum'})
    
    flag1                   = player[player['dismissal_kind'].isin(['caught','bowled','lbw','stumped','caught and bowled','hit wicket'])].groupby(['id','bowler']).agg({'is_wicket':'sum'})
    flag2                   = player[ ~ player['extras_type'].isin(['legbyes','byes','penalty'])].groupby(['id','bowler']).agg({'total_runs':'sum'})
    
    final                   = pd.merge(flag1, flag2, how = 'inner', left_on= ['bowler','id'],right_on = ['bowler','id'] ).sort_values(['is_wicket','total_runs'], ascending = [False, True]).groupby(['bowler']).head(1).reset_index()
    final['bbi']            = final['is_wicket'].astype(int).astype(str) + '/' + final['total_runs'].astype(str)
    fin                     = final.copy()
    final                   = final[['bowler','bbi']].set_index('bowler')
    temp_wicket['BBI']      = final
    temp_wicket['Avg']      = round(temp_wicket['Runs']  / temp_wicket['is_wicket'],2)
    temp_wicket['SR']       = round(temp_wicket['Balls'] / temp_wicket['is_wicket'],2)
    temp_wicket['Eco']      = round( temp_wicket['Runs'] / (temp_wicket['Balls'] / 6 ),2)
    temp_wicket['5W']       = flag1[flag1['is_wicket'] == 5].groupby(['bowler']).agg({'is_wicket':'count'}).astype(int)
    
    temp_wicket             = temp_wicket.rename(columns = {'is_wicket':'Wickets'})
    temp_wicket             = temp_wicket.fillna(0) 
    temp_wicket             = temp_wicket.astype({'5W':int})
    temp_wicket             = temp_wicket[temp_wicket['Innings'] >= innings] if innings else temp_wicket 
    temp_wicket             = temp_wicket[temp_wicket['Wickets'] >= wickets ] if wickets else temp_wicket 
    temp_wicket             = temp_wicket[choice] if choice else temp_wicket[['Innings','Wickets','Balls','Runs','BBI','Avg','SR','Eco','5W']]
    

    if flag == 'BBI':
        fin                 = fin[['bowler','is_wicket','total_runs']].set_index('bowler')
        temp_wicket[['X','Y']]  = fin
        temp_wicket = temp_wicket.sort_values(['X','Y'], ascending = [False, True]).head(limit).reset_index() 
        return temp_wicket.drop(['X','Y'], axis = 1).head(limit)
        
    return temp_wicket.sort_values(flag, ascending = asc ).head(limit).reset_index()





def get_run(ball, batsman = None, flag = 'Runs', innings = None, runs = None, choice = None, limit = 3, asc = False ):
    
    player                = ball[ball['batsman'].isin(batsman) ] if batsman else ball   
    temp_runs             = player.groupby(['batsman']).agg({'batsman_runs': 'sum'})
    
    runs_match            = player.groupby(['id']).agg({'batsman_runs':'sum'}).sort_values('batsman_runs',ascending = False)
    if len(temp_runs) == 0:
        tt = pd.DataFrame(columns = [ 'batsman','Innings','Not Out','Runs','Dots','HS','Avg','BF','SR','Hundered','Fifty','Four','Six' ] )
        tt = tt[choice] if choice else tt[['Innings','Not Out','Runs','Dots','HS','Avg','BF','SR','Hundered','Fifty','Four','Six']]
        tt['batsman'] = batsman
        return tt.fillna(0) 
    temp_runs['HS']       = player.groupby(['id','batsman']).agg({'batsman_runs':'sum'}).reset_index().groupby(['batsman']).agg({'batsman_runs':'max'})
    
    temp_runs_match       = player.groupby(['id','batsman']).agg({'batsman_runs':'sum'}).reset_index()
    temp_runs['Hundered'] = temp_runs_match[temp_runs_match['batsman_runs'] >= 100 ].groupby(['batsman']).agg({'batsman_runs':'count'})
    temp_runs['Fifty']    = temp_runs_match[ (temp_runs_match['batsman_runs'] < 100) & (temp_runs_match['batsman_runs'] >= 50) ].groupby(['batsman']).agg({'batsman_runs':'count'})
    temp_runs['Innings']  = temp_runs_match.groupby(['batsman']).agg({'id':'count'})
    
    temp_runs['No']       = ball[ball['player_dismissed'].isin(batsman)].groupby(['player_dismissed']).agg({'id':'count'}) if batsman else ball.groupby(['player_dismissed']).agg({'id':'count'})
         
    temp_runs['Not Out']  = temp_runs['Innings'] - temp_runs['No']
    temp_runs['Six']      = player[ (player['batsman_runs'] == 6) & (player['non_boundary'] == 0) ].groupby(['batsman']).agg({'batsman_runs':'count'})
    temp_runs['Four']     = player[ (player['batsman_runs'] == 4) & (player['non_boundary'] == 0) ].groupby(['batsman']).agg({'batsman_runs':'count'})
    temp_runs['Dots']     = player[player['batsman_runs'] == 0 ].groupby(['batsman']).agg({'batsman_runs':'count'})
    temp_runs['BF']       = player[~ player['extras_type'].isin(['wides'])].groupby(['batsman']).agg({'id':'count'})
    temp_runs['Avg']      = round(temp_runs['batsman_runs'] /(temp_runs['No'] ),2)
    temp_runs['SR']       = round((temp_runs['batsman_runs']/temp_runs['BF'])*100,2)
    temp_runs             = temp_runs.rename(columns = {'batsman_runs':'Runs'})

    temp_runs             = temp_runs.fillna(0) 
    temp_runs             = temp_runs.astype({'Six':int,'Four':int,'Hundered':int,'Fifty':int})
    temp_runs             = temp_runs[temp_runs['Innings'] >= innings] if innings else temp_runs    
    temp_runs             = temp_runs[temp_runs['Runs'] >= runs] if runs else temp_runs
    temp_runs             = temp_runs[choice] if choice else temp_runs[['Innings','Not Out','Runs','Dots','HS','Avg','BF','SR','Hundered','Fifty','Four','Six']]
    
    
    return temp_runs.sort_values(flag, ascending = asc ).head(limit).reset_index()
    



def get_player_data(player, ball,flag = 'bowler'):
    return ball[ball[flag].isin(player) ]

def get_team_data( team, ball, player_type = 'batsman'):
    flag = 'bowling_team' if player_type == 'batsman' else 'batting_team'
    return  ball[ball[flag].isin(team) ]



def seasonal_wickets(bowler,ball, match, game):
    player = ball[ball['bowler'] == bowler ]
    match  =   match[['id','date']]
    
    player = player[(player['is_wicket'] == 1) & (player['dismissal_kind'].isin(['caught','bowled','lbw','stumped','caught and bowled','hit wicket']))]
    final  = pd.merge(player,match , how = 'inner',left_on=['id'],right_on=['id'])
    
    final  = final[['id','date','batsman']]
    final['year'] = pd.to_datetime(final['date']).dt.strftime('%Y')
    result = final.groupby(['year']).agg({'batsman' : 'count'})

    result['Game'] = game
    
    result.reset_index(inplace=True)
    result = result.rename(columns = {'index':'Year'})
    
    return result.to_dict()




def one_vs_one(batsman,ball,match, game):
    player         = ball[ball['batsman'] == batsman ]
    match          = match[['id','date']]
    final          = pd.merge(player,match , how = 'inner',left_on=['id'],right_on=['id'])
    final          = final[['id','date','batsman_runs']]
    final['wicket']= np.where((player['is_wicket'] == 1) & (player['dismissal_kind'].isin(['caught','bowled','lbw','stumped','caught and bowled','hit wicket'])),1,0)

    final          = final[['id','date','batsman_runs','wicket']]
    final['year']  = pd.to_datetime(final['date']).dt.strftime('%Y')
    result         = final.groupby(['year','batsman_runs']).agg({'batsman_runs' : 'count','wicket':'sum'})
    wicket         = final.groupby(['year']).agg({'wicket':'sum'})
    result         = result.rename({'batsman_runs':'cnt'},axis='columns')

    result.reset_index(inplace=True)
    result = result.rename(columns = {'index':'Year'})

    wicket.reset_index(inplace=True)
    wicket = wicket.rename(columns = {'index':'Year'})
    
    result = result[['year','cnt','batsman_runs']]
    
    result = result.groupby(['year','batsman_runs']).mean().squeeze().unstack().add_suffix('')
    result = result.fillna(0)
    
    result['runs'] = 0
    result['balls'] = 0
    
    for i in ['0','1','2','3','4','5','6','7','8']:
        if i not in result.columns:
            result[i] = 0
        else:
            result['balls'] += result[i]
            result['runs'] += int(i)*result[i]
    
    result.reset_index(inplace=True)  
    result = result[['year','0','1','2','4','6','runs','balls']]
    
    result = result.rename(columns= {'0':'Dots'})
    result['Strike Rate'] = (result['runs'] / result['balls'])*100
            
    result['Game'] = game
    result = pd.merge(result,wicket,how='inner',left_on='year',right_on='year')
    
    return result, result['runs'].sum()




def decide_batsman( ball, match,team ,player = None, runs = None, opp = None, venue = None, thres = None):
    win_percent = 0
    if player:
        ball = ball[ball['batsman'] == player]    
    else:
         ball = ball[ball['batting_team'].isin(team) ]
        
    if opp != [None]:
        ball = ball[ball['bowling_team'].isin(opp) ]
        
    match_runs = ball.groupby(['id','batsman']).agg({'batsman_runs':'sum'})
    match_runs.reset_index(inplace = True)
    
    if venue:
        match = match[match['venue'] == venue]
        
    match = match[['id','winner']]
    
    final = pd.merge(match_runs, match, how= 'inner', left_on = 'id',right_on = 'id')
        
    final = final [final['batsman_runs'] >= runs]
        
    wins  = final[final['winner'].isin(team)] 
    if len(wins):
        win_percent = len(wins)/ len(final)
    
    final['result'] = np.where((final['winner'].isin(team) ),1,0)

    final = final.groupby(['batsman']).agg({'result':'sum','id':'count'})
    final['Win Percent'] = final['result'] / final['id']

    if thres:
        final = final[final['id'] >= thres]

    final = final.rename(columns = {'result':'Wins','id':'Total'})
    final = final.sort_values('Win Percent', ascending = False)

    return win_percent,final




def decide_bowler( ball, match,team ,player = None, wickets = 1, opp = None, venue = None, thres = None):
    
    win_percent = 0
    if player:
        ball = ball[ball['bowler'] == player]    
    else:
         ball = ball[ball['bowling_team'].isin(team) ]
        
    if opp != [None]:
        ball = ball[ball['batting_team'].isin(opp) ]
        
    
    ball['wicket']= np.where((ball['is_wicket'] == 1) & (ball['dismissal_kind'].isin(['caught','bowled','lbw','stumped','caught and bowled','hit wicket'])),1,0)
        
    match_runs = ball.groupby(['id','bowler']).agg({'wicket':'sum'})
    match_runs.reset_index(inplace = True)
    
    if venue:
        match = match[match['venue'] == venue]
        
    match = match[['id','winner']]
    
    final = pd.merge(match_runs, match, how= 'inner', left_on = 'id',right_on = 'id')
        
    final = final [final['wicket'] >= wickets ]
        
    wins  = final[final['winner'].isin(team)] 
    
    if len(wins):
        win_percent = len(wins)/ len(final)
    
    final['result'] = np.where((final['winner'].isin(team)),1,0)

    final = final.groupby(['bowler']).agg({'result':'sum','id':'count'})
    final['Win Percent'] = final['result'] / final['id']

    if thres:
        final = final[final['id'] >= thres]

    final = final.rename(columns = {'result':'Wins','id':'Total'})
    final = final.sort_values('Win Percent', ascending = False)

    return win_percent,final




def best_in_over(ball,decider_main, balls = 5 ):
    if decider_main in ['Runs','Wickets']:
        player_type = 'batsman' if decider_main == 'Runs' else 'bowler'
        decider = 'batsman_runs' if player_type == 'batsman' else 'is_wicket'

        if decider == 'is_wicket':
            ball = ball[(ball['is_wicket'] == 1) & (ball['dismissal_kind'].isin(['caught','bowled','lbw','stumped','caught and bowled','hit wicket']))]

        final  =  ball.groupby(['over',player_type]).agg({decider:'sum'}).reset_index()
        result = final.groupby(['over']).agg({decider:'max'}).reset_index()

        data = pd.merge(final, result, how = 'inner', left_on = ['over',decider],right_on = ['over',decider])
        
        if decider == 'is_wicket':
            data  = data.rename(columns = {'is_wicket':'Wickets'})
            data  = data.astype({'Wickets':'int'})
            
        else:
            data  = data.rename(columns = {'batsman_runs':'Runs'})
        
    elif decider_main in ['Six','Four'] :
        decider = 6 if decider_main == 'Six' else 4
        
        ball = ball.loc[ (ball['batsman_runs'] == decider ) & (ball['non_boundary'] == 0) ]
        
        final  =  ball.groupby(['over','batsman']).agg({'id':'count'}).reset_index()
        result = final.groupby(['over']).agg({'id':'max'}).reset_index()

        data = pd.merge(final, result, how = 'inner', left_on = ['over','id'],right_on = ['over','id'])
        data = data.rename(columns = {'id':decider_main})
        
    elif decider_main in ['SR'] :
        
        flag1  =  ball.groupby(['over','batsman']).agg({'batsman_runs':'sum'}).reset_index()
        flag2  =  ball[~ ball['extras_type'].isin(['wides'])].groupby(['batsman','over']).agg({'id':'count'})
        final  =  pd.merge(flag1, flag2, how = 'inner', left_on = ['over','batsman'], right_on = ['over','batsman'] )
        
        final['SR'] = round((final['batsman_runs'] / final['id']) * 100,2)
        
        final = final[final['id'] >= balls ]
        
        result = final.groupby(['over']).agg({'SR':'max'}).reset_index()
        data = pd.merge(final, result, how = 'inner', left_on = ['over','SR'],right_on = ['over','SR'])
        data = data.rename(columns = {'batsman_runs':'Runs', 'id': 'BF' })
    data = data.set_index('over')
    return data


import streamlit as st
import altair as alt
import numpy as np
import pandas as pd
import os
import matplotlib.pyplot as plt
import seaborn as sns
sns.set()
import chart_studio.plotly as py
import cufflinks as cf
from plotly.offline import download_plotlyjs, init_notebook_mode, plot, iplot
cf.go_offline()
import plotly.express as px
import plotly.graph_objs as go

absolute_path = os.path.abspath(__file__)
path = os.path.dirname(absolute_path)

matches = pd.read_csv(path+'/matches.csv')
deliveries = pd.read_csv(path+'/deliveries.csv')
matches.replace({'Sunrisers Hyderabad':'Hyderabad (Sunriser/Chargers)','Deccan Chargers':'Hyderabad (Sunriser/Chargers)',\
'Rising Pune Supergiants':'Pune (Supergiant/ Warriors)','Delhi Daredevils':'Delhi (Capitals/ Daredevils)',\
'Delhi Capitals':'Delhi (Capitals/ Daredevils)','Pune Warriors':'Pune (Supergiant/ Warriors)','Punjab Kings':'Kings XI Punjab',
'Rising Pune Supergiant':'Pune (Supergiant/ Warriors)'}, inplace= True)

@st.cache(hash_funcs={dict: lambda _: None})
def make_fig():
    some_fig = Achart().graph1()
    cached_dict = {'f1': some_fig}    
    return cached_dict



def get_team1_name():
    return np.unique(matches['team1'].values) 
def get_team2_name():
    return np.unique(matches['team2'].values) 

    

def comparison(team1,team2):
    compare = matches[((matches['team1']==team1) | (matches['team2']==team1)) & ((matches['team1']==team2) | (matches['team2']==team2))]
    fig = plt.figure(figsize=(10,5))
    sns.countplot(x='season',hue='winner',data=compare)    
    return st.pyplot(fig)
    
def lucky(matches,team):
    return matches[matches['winner']==team]['venue'].value_counts()

 
def geth2h(team1,team2):
    teams = [team1,team2]
    if team1 in teams and team2 in teams:
        return True
    else:
        return False

def winper():
    winloss = matches[['team1','team2','winner']]
    winloss.head()
    winloss['loser'] = winloss.apply(lambda x: x['team2'] if x['team1']== x['winner'] else x['team1'], axis = 1)
    teamwins = winloss['winner'].value_counts()
    teamloss = winloss['loser'].value_counts()
    played = (matches['team1'].value_counts() + matches['team2'].value_counts()).reset_index()
    played.columns = ['team','played']
    wins = matches['winner'].value_counts().reset_index()
    wins.columns = ['team','won']
    played = played.merge(wins, left_on='team', right_on='team', how='inner')
    loss = winloss['loser'].value_counts().reset_index()
    loss.columns = ['team','lost']
    played = played.merge(loss, left_on = 'team', right_on = 'team', how='inner')
    played['%win'] = round((played['won'] / played['played'])*100,2)
    played['%loss'] = round((played['lost'] / played['played']) * 100,2)
    played = played.sort_values(by='%win',ascending=False)
    return played
    
    

    
    #return h2h
    
    #fig = plt.figure(figsize=(10,5))
    #sns.countplot(x='winner',hue='df',data=h2h) 
    



import subprocess                                                               
import os
absolute_path = os.path.abspath(__file__)
                                                            
def main():                                                                   
    cmd =['streamlit', 'run', os.path.dirname(absolute_path)+'/app.py', '--server.port','5000','--theme.base','dark']                                                                 
    subprocess.run(cmd)                                                    



import streamlit as st
from teamutils import *
import altair as alt
import numpy as np
import pandas as pd
import os
import matplotlib.pyplot as plt
import seaborn as sns
sns.set()
import chart_studio.plotly as py
import cufflinks as cf
from plotly.offline import download_plotlyjs, init_notebook_mode, plot, iplot
cf.go_offline()
import plotly.express as px
import plotly.graph_objs as go

absolute_path = os.path.abspath(__file__)
path = os.path.dirname(absolute_path)

matches = pd.read_csv(path+'/matches.csv')
deliveries = pd.read_csv(path+'/deliveries.csv')
season_list = ['2007/08','2009','2009/10','2011','2012','2013','2014','2015','2016','2017','2018','2019','2020/21','2021']
season_dict = {2008:'2007/08',2009:'2009',2010:'2009/10',2011:'2011',2012:'2012',2013:'2013',2014:'2014',2015:'2015',2016:'2016',2017:'2017',2018:'2018',2019:'2019',2020:'2020/21',2021:'2021'}
team_dict   =  { 'Delhi Capitals':'Delhi Daredevils' , 'Punjab Kings':'Kings XI Punjab' }

@st.cache(max_entries=130, ttl=3600)
def make_fig():
    some_fig = Achart().graph1()
    return some_fig
    
def compare():
    st.title("Comparison Between Teams by Wins")
    team1 = st.sidebar.selectbox('Team 1', get_team1_name())
    team2 = st.sidebar.selectbox('Team 2', get_team2_name())
    comparison(team1,team2)
    
def luckvenue():
    st.title("Lucky Venue for the Team")
    team = st.sidebar.selectbox('Team', get_team1_name())
    fig=lucky(matches,team)[:10].figure(kind='bar')
    st.plotly_chart(fig)

def tosswins():
    st.title("Toss winning Decision")
    tossdec = matches.groupby('season')['toss_decision'].value_counts()
    tossdec = pd.DataFrame(tossdec)
    tossdec.columns = ['Count']
    tossdec.reset_index(inplace=True)
    tossdec.set_index('season',inplace=True)
    fig = plt.figure(figsize=(10,5))
    sns.countplot(x='season',hue='toss_decision',data=matches);
    st.pyplot(fig)
    
def wincount():
    st.title("Winning and Losing Percentage between each team")
    st.table(winper())



def totalruns():
    st.title("Total runs scored")
    batting_team_gp = deliveries.groupby(['batting_team']).total_runs.sum()
    #convert grouped data to dataframe
    batting_team_gp_df = pd.DataFrame(batting_team_gp)
    batting_team_gp_df.reset_index(inplace=True) 
    batting_team_gp_df.columns = ['Batting Team', 'Total Runs'] #rename the columns 
    fig = px.bar(batting_team_gp_df, x='Batting Team', y='Total Runs', text='Total Runs', color='Total Runs', height = 550,labels={'Batting Team':'Batting Team', 'Total Runs':'Total Runs'}, title='Total Runs by each Batting Team')
    fig.update_traces(texttemplate='%{text}', textposition='outside')
    fig.update_layout(uniformtext_minsize=8, uniformtext_mode='hide')
    fig.update_xaxes(tickangle=45)
    #figu = fig.show()
    st.plotly_chart(fig)
    
    
    
 



import streamlit as st
import pandas as pd
from utils import *
from modules import *
import numpy as np
import altair as alt
from teamutils import *
from teammodules import *

select=st.sidebar.selectbox('Cricket Analysis',[ 'Player Stats','Team Stats','Win Prediction'])

if select=='Team Stats':
    select = st.sidebar.selectbox('Team Analysis', ['Toss Wins','Lucky Venues','Comparison between Teams','Total Runs', 'Win Count' ])
    if select == 'Comparison between Teams':
        compare()
    elif select == 'Lucky Venues':
        luckvenue()
    elif select == 'Total Runs':
        totalruns()
    elif select == 'Toss Wins':
        tosswins()
    elif select == 'Win Count':
        wincount()

elif select == 'Player Stats' :
    select = st.sidebar.selectbox('Analysis', ['Player Career','Overall Stats','Seasonal Stats', 'One Vs One','Over Stats' ])
    if select == 'Player Career':
        player_career()
    elif select == 'Seasonal Stats':
        sesonal_stat()
    elif select == 'One Vs One':
        one_vs_one()
    elif select == 'Overall Stats':
        overall_stat()
    elif select == 'Over Stats':
        over_stats()
else:
    win_predict_player()

    


# runs   = pd.concat([season_runs(player, ipl_ball, ipl_match, 'IPL'), season_runs(player, bbl_ball, bbl_match, 'BBL')], ignore_index=True)
# wicket = pd.concat([season_wickets(player, ipl_ball, ipl_match, 'IPL'), season_wickets(player, bbl_ball, bbl_match, 'BBL')], ignore_index=True)


# c  =  alt.Chart(runs).mark_bar().properties(width=350).encode(
#         x='year',
#         y='batsman_runs',
#         color='Game',
#         column='Game'
#     )
# st.subheader('Batting Career')
# st.altair_chart(c)
# col1, col2 = st.columns(2)

# c  =  alt.Chart(wicket).mark_bar().properties(width=350).encode(
#         x='year',
#         y='batsman',
#         color='Game',
#         column='Game'
#     )

# st.subheader('Bowling Career')
# st.altair_chart(c)

# st.subheader('Head-Head')
# convert_dict = {'Dots': int ,'1':int,'2': int,'4': int,'6': int ,'runs':int,'balls': int,'Strike Rate':int}  

# if player_type == 'bowler':
#     batsman = st.selectbox('Player', get_player(ipl_ball,'batsman'))
#     st.subheader(player+' Vs '+batsman)

#     stat  = pd.concat([one_vs_one(batsman,get_data(player,ipl_ball,'bowler') , ipl_match, 'IPL'), 
#                        one_vs_one(batsman, get_data(batsman,bbl_ball,'bowler'), bbl_match, 'BBL')], 
#                        ignore_index=True)

# else:
#     bowler  = st.selectbox('Player', get_player(ipl_ball,'bowler'))
#     st.subheader(player+' Vs '+ bowler)

#     stat  = pd.concat([one_vs_one(player,get_data(bowler,ipl_ball,'bowler') , ipl_match, 'IPL'), 
#                        one_vs_one(player, get_data(bowler,bbl_ball,'bowler'), bbl_match, 'BBL')], 
#                        ignore_index=True)


# stat = stat[['year','Dots','1','2','4','6','runs','balls','wicket','Strike Rate','Game']]
# stat = stat.set_index('year')
# st.dataframe(stat.astype(convert_dict)  )



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['ipl_stats']

package_data = \
{'': ['*']}

install_requires = \
['altair>=4.2.0,<5.0.0',
 'chart-studio>=1.1.0,<2.0.0',
 'cufflinks>=0.17.3,<0.18.0',
 'importlib-resources>=5.4.0,<6.0.0',
 'matplotlib>=3.5.1,<4.0.0',
 'pandas>=1.4.1,<2.0.0',
 'plotly>=5.6.0,<6.0.0',
 'seaborn>=0.11.2,<0.12.0',
 'streamlit>=1.5.1,<2.0.0']

entry_points = \
{'console_scripts': ['start = src.ipl_stats.runserver:main']}

setup_kwargs = {
    'name': 'ipl-stats',
    'version': '0.1.2.2',
    'description': '',
    'long_description': None,
    'author': 'naveengct',
    'author_email': 'naveenrajesh2222@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
